```@setup from_csv
include("../../example/from_csv.jl")
```

# Tutorials

## Manual construction
If you know the coefficients of the Sellmeier equation already, you can construct a corresponding `SellmeierModel` directly.  Here is an example for a three-term Sellmeier equation:

```julia
str = [0.696, 0.408, 0.897]  # strength of oscillators (B₁,B₂,B₃)
λres = [0.0684e-6, 0.116e-6, 9.896e-6]  # resonance wavelengths in meters (√C₁,√C₂,√C₃)
mdl = SellmeierModel(str, λres)
```

Once a `SellmeierModel` is constructed, the Sellmeier equation represented by the model can be evaluated by

```julia
mdl(1.55e-6)  # ε (or n²) at λ = 1.55 µm
```

If you would like to construct a Sellmeier model for a nondispersive material (which has a constant refractive index ``n`` for all wavelengths), simply call the constructor `SellmeierModel` with the constant dielectric constant ``ε_\mathrm{r} = n²`` as a sole argument:

```julia
εconst = 4.0  # n = √ε = 2.0
mdl = SellmeierModel(εconst)  # mdl(λ) returns εconst for any λ
```

## Construction by fitting data to the Sellmeier equation
If you have measured dielectric constant data at various wavelengths, you can construct `SellmeierModel` by fitting the data to the Sellmeier equation.  Suppose `ε` is a vector of measured dielectric constants and `λ` is a vector of measurement wavelengths in meters.  Fitting these data to the Sellmeier equation is performed by

```julia
(; mdl, err) = fit_sellmeier(λ, ε)
```

Here, `mdl` is an instance of `SellmeierModel` that has `str` and `λres` as fields; they represent the strengths and resonance wavelengths of the terms in the Sellmeier equation, and correspond to ``B_i``'s and ``\sqrt{C_i}``'s, resectively.  `err` is the root-mean-square error of the entries of `ε` with respect to the fitting Sellmeier equation.

The above code determines the optimal number ``N`` of terms in the Sellmeier equation.  There are a few variant usages of `fit_sellmeier()`.  If you would like to find the optimal Sellmeier equation with a specific number of terms, you can specify ``N`` manually by

```julia
(; mdl, err) = fit_sellmeier(λ, ε, N)
```

Furthermore, you can specify the numbers ``N_n`` and ``N_p`` of terms with ``\sqrt{C_i}`` below and above the wavelength range of measurement, respectively, by

```julia
(; mdl, err) = fit_sellmeier(λ, ε, Nₙ, Nₚ)
```

Note that ``\sqrt{C_i}``'s in the Sellmeier equation should lie either below or above the wavelength range of measurement, as discussed in [How SellmeierFit.jl works](@ref).  Hence ``N_n + N_p = N``.

## Loading refractive index data
The refractive index data to construct a Sellmeier equation with are often stored in a file.  `SellmeierFit.read()` is a convenience function that reads refractive index data from a file and outputs wavelengths ``λ`` and dielectric constants ``ε_\mathrm{r}`` to pass to the `SellmeierModel` constructor discussed above.  The function supports various keyword arguments to read refractive index data in various formats.  See [`SellmeierFit.read()`](@ref) for the details of available keyword arguments.

### From a CSV file
Suppose a CSV file named `refindex.csv` in the present working directory contains wavelengths ``λ`` in units of µm and the corresponding refractive indices ``n`` as the first and second columns, respectively.  The following code stores wavelengths in units of meters (not µm) and dielectric constants in variables `λ` and `ε` via [property destructuring of `NamedTuple`](https://julialang.org/blog/2021/11/julia-1.7-highlights/#property_destructuring):

```julia
(; λ, ε) = SellmeierFit.read("refindex.csv")
```

`λ` and `ε` can be equally obtained by

```julia
result = SellmeierFit.read("refindex.csv")
λ = result.λ
ε = result.ε
```

or

```julia
result = SellmeierFit.read("refindex.csv")
λ = result[1]
ε = result[2]
```

Only the first method using property destructuring will be discussed in the following sections, but the other methods are applicable there as well.

### From a text file
Suppose a text file named `refindex.txt` contains wavelengths in units of µm and the corresponding refractive indices as follows:

```
Refractive Index
wl 	n
0.21	1.53835762
0.2174	1.530846431
0.2251	1.524078907
0.233	1.518041768
0.2412	1.512572116
...
```

In this case, the information that the first two lines of the file are a header needs to be provided to `SellmeierFit.read()`:

```julia
(; λ, ε) = SellmeierFit.read(filepath, header=2)
```

### From RefractiveIndex.INFO
[RefractiveIndex.INFO](https://RefractiveIndex.INFO) is a website collecting a vast amount of experimentally measured refractive index data published in literature.  `SellmeierFit` provides a convenience method `RefractiveIndexInfo()` that downloads the refractive index data in CSV format from RefractiveIndex.INFO and store them locally for future use:

```julia
filepath = RefractiveIndexInfo(material, author)
(; λ, ε) = SellmeierFit.read(filepath)
```

The arguments `material` and `author` correspond to the chemical formula of the material and the surname of the first author of the publication containing the measured refractive index data.  For example, `material = "SiO2"` and `auhtor = "Malitson"` are used to retrieve the data presented at [`https://refractiveindex.info/?shelf=main&book=SiO2&page=Malitson`](https://refractiveindex.info/?shelf=main&book=SiO2&page=Malitson), and the `book` and `page` fields at the end of the URL provide the strings to use for our `material` and `author`.

The file is downloaded in CSV format.  The return value `filepath` is the local path of the downloaded CSV file.  The default download directory is `RefractiveIndex.INFO/<material>/` in the SellmeierFit package root directory, and the file is named `<author>.csv`.  The download directory and file name can be changed by passing the keyword arguments `dir` and `file`; see [`SellmeierFit.RefractiveIndexInfo()`](@ref) for the details.

!!! note
    The download is performed only when the file does not exist already in the download directory.  If the file already exists, then the path to the existing file is returned as `filepath`.

### Loading and fitting data at once
`fit_sellmeier()` can take a file path instead of `λ` and `ε`, such that you can load and fit the data in one line:

```julia
(; mdl, err) = fit_sellmeier(filepath)
(; mdl, err) = fit_sellmeier(RefractiveIndexInfo(material, author))
```

A downside of this method is that the loaded `λ` and `ε` are not stored in separate variables: you only get `mdl::SellmeierModel`.  If you need to examine `λ` and `ε`, load the data separately by `(; λ, ε) = SellmeierFit.read(...)`.

## Concrete examples
Concrete example scripts are included in the `example/` directory.  Here is the plot generated by `example/from_csv.jl`:

```@example from_csv
save("SiO₂.png", fig)  # hide
nothing  # hide
```
![SiO₂](SiO₂.png)

Note that the fitting Sellmeier equation passes through all the data points almost exactly.  The calculated fitting parameters as shown in the output of `example/from_csv.jl` are

```@example from_csv
println("B = $(mdl.str)")  # hide
println("√C (µm) = $(mdl.λres ./ micro)")  # hide
```

and agree well with the values shown in the Dispersion formula section [here](https://refractiveindex.info/?shelf=main&book=SiO2&page=Malitson).
